#include "GG_MiniCar.h"

DC_Car::DC_Car(int Left_Pin1,int Left_Pin2,bool Left_Reverse,int Right_Pin1,int Right_Pin2,bool Right_Reverse,bool All_PWM_Pin) {
	this->_CarAllPWMPin=All_PWM_Pin;
//以下左輪 =========================================================================
	this->_LeftReverse=Left_Reverse;
	if (this->_LeftReverse && this->_CarAllPWMPin) {
		this->_LF_Pin=Left_Pin2;
		this->_LB_Pin=Left_Pin1;
	} else {
		this->_LF_Pin=Left_Pin1;
		this->_LB_Pin=Left_Pin2;
	}
//以下右輪 =========================================================================
	_RightReverse=Right_Reverse;
	if (_RightReverse && this->_CarAllPWMPin) {
		this->_RF_Pin=Right_Pin2;
		this->_RB_Pin=Right_Pin1;
	} else {
		this->_RF_Pin=Right_Pin1;
		this->_RB_Pin=Right_Pin2;
	}
//以下基礎參數 =======================================================================
  this->_rate=DefaultSpeed;
  this->_dc_SFC=0;
  this->_Left_SFC=0;
  this->_Right_SFC=0;
  this->_LeftPWM=DefaultSpeed;
  this->_RightPWM=DefaultSpeed;
	this->_MaxSetSpeed=GG_MiniCar_MaxSpeed;
	this->_MinSetSpeed=0;
}

void DC_Car::Drive(bool left_forward,bool left_backward,bool right_forward,bool right_backward) {
//以下左輪 =========================================================================
  pinMode(this->_LF_Pin,OUTPUT);
	if (this->_CarAllPWMPin) {
	  digitalWrite(this->_LF_Pin,left_forward);
	} else {
	  digitalWrite(this->_LF_Pin,this->_LeftReverse ? !left_forward : left_forward);
	}
  pinMode(this->_LB_Pin,OUTPUT);
  digitalWrite(this->_LB_Pin,left_backward);
//以下右輪 =========================================================================
  pinMode(this->_RF_Pin,OUTPUT);
	if (this->_CarAllPWMPin) {
	  digitalWrite(this->_RF_Pin,right_forward);
	} else {
	  digitalWrite(this->_RF_Pin,_RightReverse ? !right_forward : right_forward);
	}
  pinMode(this->_RB_Pin,OUTPUT);
  digitalWrite(this->_RB_Pin,right_backward);
}

void DC_Car::pwmDrive(int left_forward,int left_backward,int right_forward,int right_backward) {
//以下左輪 =========================================================================
	if (this->_CarAllPWMPin) {
		analogWrite(this->_LF_Pin,left_forward);
	} else {
		pinMode(this->_LF_Pin,OUTPUT);
	  digitalWrite(this->_LF_Pin,this->_LeftReverse ? !left_forward : left_forward);
	}
  analogWrite(this->_LB_Pin,left_backward);
//以下右輪 =========================================================================
	if (this->_CarAllPWMPin) {
		analogWrite(this->_RF_Pin,right_forward);
	} else {
		pinMode(this->_RF_Pin,OUTPUT);
	  digitalWrite(this->_RF_Pin,_RightReverse ? !right_forward : right_forward);
	}
  analogWrite(this->_RB_Pin,right_backward);
}

void DC_Car::SetSpeed(int speed) {
	if (speed<this->_MinSetSpeed) {	//小於最小值
		this->_rate=this->_MinSetSpeed;
		Serial.print("Error! Change speed to Minimum Speed=");
		Serial.println(this->_MinSetSpeed);
	} else if (speed>this->_MaxSetSpeed) {	//大於最大值
		this->_rate=this->_MaxSetSpeed;		
		Serial.print("Error! Change speed to maximum Speed=");
		Serial.println(_MaxSetSpeed);
	} else {
		this->_rate=speed;
	}
	this->_LeftPWM=this->_rate+this->_Left_SFC;	//左輪速率=設定速率+左輪直線修正值
	this->_RightPWM=this->_rate-this->_Right_SFC;	//左輪速率=設定速率-右輪直線修正值
}

void DC_Car::SetStraightAdj(int adj) {
	int reminder;
	if (adj>MaxStraightForwardCorrection) {	//比最大值大
		this->_dc_SFC=MaxStraightForwardCorrection;
	} else if (adj<MinStraightForwardCorrection) {	//比最小值小
		this->_dc_SFC=MinStraightForwardCorrection;
	} else {
		this->_dc_SFC=adj;
	}
	reminder=this->_dc_SFC % 2;
	this->_Right_SFC=this->_dc_SFC / 2.0;	//左輪調整一半
	this->_Left_SFC=this->_Right_SFC+reminder;	//右輪調整一半+一半的餘數
	this->_rate=DefaultSpeed;	//速率設為預設值
	this->_MaxSetSpeed=GG_MiniCar_MaxSpeed-abs(this->_Right_SFC);	//可設定的最大速率
	this->_MinSetSpeed=abs(this->_Right_SFC);	//可設定的最小速率
	this->_LeftPWM=this->_rate+this->_Left_SFC;	//設定左輪轉速	
	this->_RightPWM=this->_rate-this->_Right_SFC;	//設定右輪轉速
}

void DC_Car::pwmForward() {
	if (this->_CarAllPWMPin) {
		pwmDrive(this->_LeftPWM,0,this->_RightPWM,0);
	} else {
		pwmDrive(HIGH,this->_LeftPWM,HIGH,this->_RightPWM);
	}
}

void DC_Car::pwmBackward() {
	if (this->_CarAllPWMPin) {
		pwmDrive(0,this->_LeftPWM,0,this->_RightPWM);
	} else {
		pwmDrive(LOW,this->_LeftPWM,LOW,this->_RightPWM);
	}
}

void DC_Car::pwmTurnLeftLow() {
	if (this->_CarAllPWMPin) {
		pwmDrive(0,0,this->_RightPWM,0);
	} else {
		pwmDrive(HIGH,0,HIGH,this->_RightPWM);
	}
}

void DC_Car::pwmTurnLeftHigh() {
	if (this->_CarAllPWMPin) {
		pwmDrive(0,this->_LeftPWM,this->_RightPWM,0);
	} else {
		pwmDrive(LOW,this->_LeftPWM,HIGH,this->_RightPWM);
	}
}

void DC_Car::pwmTurnLeftArc(int sDifference) {
	if (sDifference>this->_LeftPWM) sDifference=this->_LeftPWM;
	if (this->_CarAllPWMPin) {
		pwmDrive(this->_LeftPWM-sDifference,0,this->_RightPWM,0);
	} else {
		pwmDrive(HIGH,this->_LeftPWM-sDifference,HIGH,this->_RightPWM);
	}
}

void DC_Car::pwmTurnRightLow() {
	if (this->_CarAllPWMPin) {
		pwmDrive(this->_LeftPWM,0,0,0);
	} else {
		pwmDrive(HIGH,this->_LeftPWM,HIGH,0);
	}
}

void DC_Car::pwmTurnRightHigh() {
	if (this->_CarAllPWMPin) {
		pwmDrive(this->_LeftPWM,0,0,this->_RightPWM);
	} else {
		pwmDrive(HIGH,this->_LeftPWM,LOW,this->_RightPWM);
	}
}

void DC_Car::pwmTurnRightArc(int sDifference) {
	if (sDifference>this->_RightPWM) sDifference=this->_RightPWM;
	if (this->_CarAllPWMPin) {
		pwmDrive(this->_LeftPWM,0,this->_RightPWM-sDifference,0);
	} else {
		pwmDrive(HIGH,this->_LeftPWM,HIGH,this->_RightPWM-sDifference);
	}
}

void DC_Car::pwmStop() {
  pwmDrive(0,0,0,0);
}

void DC_Car::pwmWait() {
	if (this->_CarAllPWMPin) {
		pwmDrive(GG_MiniCar_MaxSpeed,GG_MiniCar_MaxSpeed,GG_MiniCar_MaxSpeed,GG_MiniCar_MaxSpeed);
	} else {
		pwmDrive(255,0,255,0);
	}
}

void DC_Car::SetJoystickMiddle(int MiddleX,int MiddleY,int Radius) {
	this->_JoystickMiddleX=MiddleX;
	this->_JoystickMiddleY=MiddleY;
	this->_JoystickRadius=Radius;
}

void DC_Car::JoystickDrive(int StickX,int StickY) {
	int DirX,DirY;
	float PWMRatio,TurnRatio;

	DirY=this->_JoystickMiddleY-StickY;	//上->下，>0為上,上=0,下=255
	DirX=this->_JoystickMiddleX-StickX;	//左->右，>0為左,左=0,右=255
	if (abs(DirY)>=abs(DirX)) {
	  PWMRatio=abs((float)DirY/this->_JoystickRadius);
	  TurnRatio=PWMRatio-abs((float)DirX/this->_JoystickRadius*PWMRatio);
	} else {
	  PWMRatio=abs((float)DirX/this->_JoystickRadius);
	  TurnRatio=abs((float)DirY/this->_JoystickRadius*PWMRatio);
	}
	if (DirY>=0) {	//前進
  	if (this->_CarAllPWMPin) {
			if (DirX>=0) {	//左轉
				pwmDrive(int(this->_LeftPWM*TurnRatio),0,int(this->_RightPWM*PWMRatio),0);
			} else {				//右轉
				pwmDrive(int(this->_LeftPWM*PWMRatio),0,int(this->_RightPWM*TurnRatio),0);
			}
		} else {
			if (DirX>=0) {	//左轉
				pwmDrive(1,int(this->_LeftPWM*TurnRatio),1,int(this->_RightPWM*PWMRatio));
			} else {				//右轉
				pwmDrive(1,int(this->_LeftPWM*PWMRatio),1,int(this->_RightPWM*TurnRatio));
			}
		}	
	} else {				//後退
		if (DirX>=0) {	//左轉
			pwmDrive(0,int(this->_LeftPWM*TurnRatio),0,int(this->_RightPWM*PWMRatio));
		} else {					//右轉
			pwmDrive(0,int(this->_LeftPWM*PWMRatio),0,int(this->_RightPWM*TurnRatio));
		}
	}
}

void DC_Car::Forward() {
	if (this->_CarAllPWMPin) {
		Drive(HIGH,LOW,HIGH,LOW);
	} else {
		Drive(HIGH,HIGH,HIGH,HIGH);
	}
}

void DC_Car::Backward() {
	Drive(LOW,HIGH,LOW,HIGH);
}

void DC_Car::TurnLeftLow() {
	if (this->_CarAllPWMPin) {
		Drive(LOW,LOW,HIGH,LOW);
	} else {
		Drive(LOW,LOW,HIGH,HIGH);
	}
}

void DC_Car::TurnLeftHigh() {
	if (this->_CarAllPWMPin) {
		Drive(LOW,HIGH,HIGH,LOW);
	} else {
		Drive(LOW,HIGH,HIGH,HIGH);
	}
}

void DC_Car::TurnRightLow() {
	Drive(HIGH,LOW,LOW,LOW);
}

void DC_Car::TurnRightHigh() {
	Drive(HIGH,LOW,LOW,HIGH);
}

void DC_Car::Stop() {
	Drive(LOW,LOW,LOW,LOW);
}

void DC_Car::Wait() {
	if (this->_CarAllPWMPin) {
		Drive(HIGH,HIGH,HIGH,HIGH);
	} else {
		Drive(HIGH,LOW,HIGH,LOW);
	}
}
//==============================================================================
DC_Motor::DC_Motor(int in1,int in2,bool isReverse,bool All_PWM_Pin) {
	this->_DirReverse=isReverse;
	this->_MotorAllPWMPin=All_PWM_Pin;
	if (isReverse && this->_MotorAllPWMPin) {
		this->_MotorIN1=in2;
		this->_MotorIN2=in1;
	} else {
		this->_MotorIN1=in1;
		this->_MotorIN2=in2;
	}
}

void DC_Motor::Drive(bool forward,bool backward) {
  pinMode(this->_MotorIN1,OUTPUT);
	if (this->_MotorAllPWMPin) {
	  digitalWrite(this->_MotorIN1,forward);
	} else {
	  digitalWrite(this->_MotorIN1,this->_DirReverse ? !forward : forward);
	}
  pinMode(this->_MotorIN2,OUTPUT);
  digitalWrite(this->_MotorIN2,backward);
}

void DC_Motor::pwmDrive(int forward,int backward) {
	if (this->_MotorAllPWMPin) {
		analogWrite(this->_MotorIN1,forward);
	} else {
		pinMode(this->_MotorIN1,OUTPUT);
	  digitalWrite(this->_MotorIN1,this->_DirReverse ? !forward : forward);
	}
  analogWrite(this->_MotorIN2,backward);
}
//==============================================================================
Trace::Trace(int IrPin) {
  this->_ir_pin=IrPin;
}

bool Trace::isDetectWhite() {
  pinMode(this->_ir_pin,INPUT);
  return digitalRead(this->_ir_pin);
}
//==============================================================================
Avoid::Avoid(int IrPin) {
  this->_AvoidIr_pin=IrPin;
}

bool Avoid::isDetectAvoid() {
  pinMode(this->_AvoidIr_pin,INPUT);
  return digitalRead(this->_AvoidIr_pin);
}
