#ifndef GG_MiniCar_h
#define GG_MiniCar_h

#if ARDUINO >= 100
	#include "Arduino.h"
#else
	#include "WProgram.h"
#endif

#if defined(ESP32)
	#include "analogWrite.h"
#endif

#define MaxStraightForwardCorrection 128
#define MinStraightForwardCorrection -128
#define DefaultSpeed 128
#define GG_MiniCar_MaxSpeed 255

class DC_Car
{
	public:
		DC_Car(int Left_Pin1,int Left_Pin2,bool Left_Reverse,int Right_Pin1,int Right_Pin2,bool Right_Reverse,bool All_PWM_Pin);
		void Drive(bool left_forward,bool left_backward,bool right_forward,bool right_backward);
		void pwmDrive(int left_forward,int left_backward,int right_forward,int right_backward);
		void SetSpeed(int speed);
		void SetStraightAdj(int adj);
		void pwmForward();
		void pwmBackward();
		void pwmTurnLeftLow();
		void pwmTurnLeftHigh();
		void pwmTurnLeftArc(int sDifference);
		void pwmTurnRightLow();
		void pwmTurnRightHigh();
		void pwmTurnRightArc(int sDifference);
		void pwmStop();
		void pwmWait();
		void Forward();
		void Backward();
		void TurnLeftLow();
		void TurnLeftHigh();
		void TurnRightLow();
		void TurnRightHigh();
		void Stop();
		void Wait();
		void JoystickDrive(int StickX,int StickY);
		void SetJoystickMiddle(int MiddleX,int MiddleY,int Radius);

	private:
		int _LF_Pin,_LB_Pin,_RF_Pin,_RB_Pin;
		int _rate=DefaultSpeed,_LeftPWM=DefaultSpeed,_RightPWM=DefaultSpeed;
		int _MaxSetSpeed=GG_MiniCar_MaxSpeed,_MinSetSpeed=0;//直線修正後,可以設定的速率範圍
		int _dc_SFC=0,_Left_SFC=0,_Right_SFC=0; //車子直進校正
		int _JoystickRadius=127,_JoystickMiddleX=128,_JoystickMiddleY=127;
		bool _CarAllPWMPin=true,_LeftReverse=false, _RightReverse=false;
};

class DC_Motor
{
	public:
	DC_Motor(int in1,int in2,bool isReverse,bool All_PWM_Pin);
	void Drive(bool forward,bool backward);
	void pwmDrive(int forward,int backward);

	private:
	int _MotorIN1,_MotorIN2;
	bool _MotorAllPWMPin=true,_DirReverse=false;
};

class Trace
{
	public:
	Trace(int IrPin);
	bool isDetectWhite();

	private:
	  int _ir_pin;
};

class Avoid
{
	public:
	Avoid(int IrPin);
	bool isDetectAvoid();

	private:
	  int _AvoidIr_pin;
};

#endif